/*
     ``The contents of this file are subject to the Mozilla Public License
     Version 1.1 (the "License"); you may not use this file except in
     compliance with the License. You may obtain a copy of the License at
     http://www.mozilla.org/MPL/

     Software distributed under the License is distributed on an "AS IS"
     basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
     License for the specific language governing rights and limitations
     under the License.

     The Original Code is SylphDS (www.sylphds.net).

     The Initial Developer of the Original Code is Ficedula (web@sylphds.net).
*/

#include "SylphActions.h"
#include <nds.h>

SylphFrameType SylphAction::DefaultType() {
	return sfDisplayConstant;
}

SylphAction::~SylphAction() {
}

SylphFrame::SylphFrame() : FLastScanLines(0), FFrameCount(0) {
}

void SylphFrame::FrameStep(bool doprogress) {
	swiWaitForVBlank();
	u32 i,j;
	int scanline = DISP_Y;
	for (i=0; i<2; ++i) {
		for(j=0; j < constant[i].size(); ++j) {
			constant[i][j]->Step(*this);
		}
		if (doprogress) 
			for(j=0; j < progress[i].size(); ++j) {
				progress[i][j]->Step(*this);
			}
	}

	while (DISP_Y == scanline) ;
	scanline = DISP_Y - scanline;
	while (scanline < 0) scanline += 263;
	FLastScanLines = scanline;
	
	FFrameCount++;
}

void SylphFrame::Register(SylphAction& action, SylphFrameType type) {
	SylphFrameType ftype = type;
	if (type == sfAuto) ftype = action.DefaultType();
	switch (ftype) {
		case sfDisplayConstant:		constant[0].push_back(&action); break;
		case sfDisplayProgress:		progress[0].push_back(&action); break;
		case sfProcessConstant:		constant[1].push_back(&action); break;
		case sfProcessProgress:		progress[1].push_back(&action); break;
		default:					printf("Not registering unknown type\n"); return;
	}
}

void SylphFrame::Unregister(SylphAction& action, SylphFrameType type) {
	SylphActions* act;
	SylphFrameType ftype = type;
	if (type == sfAuto) ftype = action.DefaultType();
	switch (ftype) {
		case sfDisplayConstant:		act = &constant[0]; break;
		case sfDisplayProgress:		act = &progress[0]; break;
		case sfProcessConstant:		act = &constant[1]; break;
		case sfProcessProgress:		act = &progress[0]; break;
		default:					return;
	}
	SylphActions::iterator it;
	for (it = act->begin(); it != act->end(); it++)
		if (*it == &action) {
			act->erase(it);
			break;
		}
}

