#ifndef __SYLPHDATA_H
#define __SYLPHDATA_H

#include <vector>
#include <string>
#include <nds.h>

enum SylphSeek {soStart, soCurrent, soEnd};

class SylphFile {
	public:
		virtual ~SylphFile();
		
		virtual int Read(void* buffer, int count)=0;
		virtual int Seek(int offset, SylphSeek Where); //default implementation just updates Position and nothing else
	public:
		std::string Filename;
		int FileSize;
		int Flags;
		int Position;
};

static const int ATTR_VALID = 	0x0001;
static const int ATTR_FOLDER = 	0x0040;

class SylphFileInfo {
	public:
		SylphFile* File();
	public:
		std::string Filename;
		int FileSize;
		u16 Attributes;
};

typedef std::vector<SylphFileInfo*> SylphFiles;

class SylphDataPlugin {
	protected:
		virtual int GetData(std::string filename, SylphFile** file)=0;
		virtual SylphFiles* EnumFiles(std::string path)=0;
	public:
		friend class SylphFiler;
		virtual ~SylphDataPlugin();
		virtual std::string PluginName()=0;
};

typedef std::vector<SylphDataPlugin*> SylphDataPlugins;

class SylphFiler {
	protected:
		SylphDataPlugins plugins;
	public:
		SylphFiler();
		~SylphFiler();
		SylphFile* GetData(std::string filename);
		SylphFiles EnumFiles(std::string path);
		void RegisterPlugin(SylphDataPlugin& plugin);
};

SylphFiler& Filer();

#endif
