/*
     ``The contents of this file are subject to the Mozilla Public License
     Version 1.1 (the "License"); you may not use this file except in
     compliance with the License. You may obtain a copy of the License at
     http://www.mozilla.org/MPL/

     Software distributed under the License is distributed on an "AS IS"
     basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
     License for the specific language governing rights and limitations
     under the License.

     The Original Code is SylphDS (www.sylphds.net).

     The Initial Developer of the Original Code is Ficedula (web@sylphds.net).
*/

#include <string>
#include "SylphData.h"
#include <malloc.h>

#include "SylphDataNet.h"
#include "SylphUDP.h"

void SylphNetFileScan::FIncoming(u16 PacketID, int datasize, u8* data) {
	int i, offset;
	if ((PacketID & 0x8000) && (FFiles.size() != (PacketID & 0x7fff))) { //start of data, and we've not initialised previously
		for (i=0; i < (int)FFiles.size(); ++i)
			delete FFiles[i];
		FFiles.clear();
		for (i=0; i < (PacketID & 0x7fff); ++i) FFiles.push_back(NULL);
//		printf("NFileScan: reserve %d files\n", FFiles.size());
	}
	if (PacketID & 0x8000) offset = 0; else offset = PacketID;
	u16 count;
	memcpy(&count, data, 2);
	data += 2;
	
	char stmp[1024];
	
	for (i=offset; i < (offset+count); ++i) {
//		printf("NFileScan: recv info file %d\n", i);
		SylphFileInfo* info = new SylphFileInfo;
		data += 4; //skip FileID
//		memcpy(&info->FileID, data, 4); data += 4; //not actually stored, or really required
//although we COULD cache this in our lookup map if we wanted
		memcpy(&info->FileSize, data, 4); data += 4;
		memcpy(&info->Attributes, data, 2); data += 2;
		u16 wtmp;
		memcpy(&wtmp, data, 2); data += 2;
		memcpy(&stmp[0], data, wtmp); data += wtmp;
		stmp[wtmp] = 0;
		info->Filename = &stmp[0];
//		printf("%s\n", stmp);
		if (FFiles[i]) delete FFiles[i];
		FFiles[i] = info;
	}
	bool complete = true;
	for (i=0; i < (int)FFiles.size(); ++i)
		if (FFiles[i] == NULL) {
			complete = false;
			break;
		}
	FComplete = complete;
}

int SylphNetDataPlugin::GetData(std::string filename, SylphFile** file) {
	printf("GetData %s\n", filename.c_str());
	int i; 
	SylphNetFileMap* map=NULL;
	for (i=0; i < maps.size(); ++i)
		if (maps[i]->Filename == filename) {
			map = maps[i];
			break;
		}
	if (map==NULL) { //TODO: Only cache a certain number of map entries
		SUDP().FileStat(filename);
		map = new SylphNetFileMap;
		map->Attributes = 0;
		map->Filename = filename;
		map->FileID = -2;
		maps.push_back(map);
		while (map->FileID == -2) {
			if (FFrameSpin) {
				FFrameSpin->FrameStep(false);
				if ((FFrameSpin->FrameCount() & 63) == 0) {
					printf("Waiting filestat %s...", filename.c_str());
					SUDP().FileStat(filename);
				}
			}
		}
	}
	printf("ID %d", map->FileID);
	if (map->FileID < 0) return 0;
	
	printf("SylphNetDataPlugin:activate file\n");
	printf("ID=%d, DataSize=%d\n", map->FileID, map->FileSize);
	printf("C %d M %d U %d\n", mallinfo().fordblks, mallinfo().arena, mallinfo().uordblks);
	SylphNetFile& nfile = *new SylphNetFile(*this);
	nfile.FFileID = map->FileID;
	nfile.FDatasize = map->FileSize;

	int chunks = (nfile.FDatasize + 1023) >> 10;
	nfile.FData = new u8*[chunks];
	if (nfile.FData == 0) printf("ARGH!");
	for (i=0; i<chunks; ++i) {
		nfile.FData[i] = new u8[1024];
		if (nfile.FData[i] == 0) printf("AAGH!");
	}
	nfile.FValidmask = new u8[(nfile.FDatasize >> 13)+1];
	for (i=0; i < ((nfile.FDatasize >> 13)+1); ++i)
		nfile.FValidmask[i] = 0; //this could be more efficient!
	nfile.FileSize = map->FileSize;
	nfile.Position = 0;
	printf("Finalise open...");
	children.push_back(&nfile);
	printf("OK\n");
	*file = &nfile;
	return 1;
}

void SylphNetDataPlugin::IncomingScan(std::string Filename, u16 PacketID, int datasize, u8* data) {
	int i;
	for (i=0; i < (int)scans.size(); ++i)
		if (scans[i]->FolderName == Filename) {
			scans[i]->FIncoming(PacketID, datasize, data);
			break;
		}
}


SylphFiles* SylphNetDataPlugin::EnumFiles(std::string path) {
	SUDP().FileScan(path);
	SylphNetFileScan scan;
	scan.FolderName = path;
	scan.FComplete = false;
	scans.push_back(&scan);

	while (!scan.FComplete) {
		if (FFrameSpin) {
			FFrameSpin->FrameStep(false);
			if ((FFrameSpin->FrameCount() & 63) == 0) {
				printf("Waiting filescan %s...", path.c_str());
				SUDP().FileScan(path);
			}
		}
	}

	if (scan.FFiles.size()) {
		SylphFiles* result = new SylphFiles;
		*result = scan.FFiles;
		return result;
	}
	return NULL;
}

SylphNetDataPlugin::SylphNetDataPlugin() {
	children.reserve(8);
	outstanding.reserve(4);
	maps.reserve(32);
}

SylphNetDataPlugin::~SylphNetDataPlugin() {
}

void SylphNetDataPlugin::IncomingData(int FileID, int offset, int datasize, u8* data) {
	SylphNetFileReqs::iterator it;
	for (it = outstanding.begin(); it != outstanding.end(); it++)
		if ((*it)->FParent.FFileID == FileID) {
			(*it)->FParent.IncomingData(offset, datasize, data);
			break;
		}
}

void SylphNetDataPlugin::FileStat(int FileID, int FileSize, u16 Attr, std::string Filename) {
	printf("FileStat: %s=%d Sz %d Attr %d", Filename.c_str(), FileID, FileSize, Attr);
	int i;
	for (i=0; i < maps.size(); ++i) {
//		printf("Match %s / %s\n", Filename.c_str(), (*it)->Filename.c_str());
		if (maps[i]->Filename == Filename) {
//			printf("-OK\n");
			maps[i]->FileSize = FileSize;
			maps[i]->Attributes = Attr;
			maps[i]->FileID = FileID;
			break;
		}
	}
}

std::string SylphNetDataPlugin::PluginName() {
	return "Wifi/UDP data access";
}

SylphNetFileReq::SylphNetFileReq(SylphNetFile& parent) : Offset(0), Range(0), Complete(0), FParent(parent) {
}

void SylphNetFileReq::Ping(bool resend) {
	int requests[256];
	int count = 0;
	int i;
	for (i=Offset; (i<(Offset+Range)) && (count < 256); ) {
		if (((FParent.FValidmask[i>>3] >> (i&7)) & 1) == 0) {
			requests[count++] = i << 10;
			int sz = 0;
			while ((((FParent.FValidmask[i>>3] >> (i&7)) & 1) == 0) && (i<(Offset+Range))) {
				sz += 1024;
				++i;
			}
			requests[count++] = sz;
		} else ++i;
	}
	if (count) {
		if (resend) {
//			printf("ReqRS OS %d R %d R0 %d\n", Offset, Range, requests[0]);
			SUDP().FileRead(FParent.FFileID, count, &requests[0]);
		}
	} else
		Complete=TRUE;
}

int SylphNetFile::Read(void* buffer, int count) {
	if ((Position + count) > FDatasize) count = FDatasize - Position;
	if (count <= 0) return 0;
//	printf("Rd %d/%d VM %d. ", Position, count, FValidmask[0]);

	int valid = 1;
	int i;
	int offset = (Position >> 10);
	int range = ((count-1) >> 10)+1;
	if ((Position>>10) != ((Position+count)>>10)) range++; //goes over a 1KB boundary
	for (i=offset; i<(offset+range); ++i)
		if (((FValidmask[i>>3] >> (i&7)) & 1) == 0) {
			valid = 0;
			break;
		}

	if (!valid) {
		//printf("SylphNetFile:req %d (%d)\n", count, Position);
		SylphNetFileReq& req = *new SylphNetFileReq(*this);
		req.Offset = (Position >> 10); //1KB block size; 
		req.Range = ((count-1) >> 10)+1;
		if ((Position>>10) != ((Position+count)>>10)) req.Range++; //goes over a 1KB boundary
		FParent.outstanding.push_back(&req);
		req.Ping(true); //initial request
		while (!req.Complete) {
			if (FParent.FFrameSpin) {
				FParent.FFrameSpin->FrameStep(false);
				if ((FParent.FFrameSpin->FrameCount() & 63)==0) {
					printf(".");
					req.Ping(true);
				} else req.Ping(false);
			}
		}
		//printf("OK");
		SylphNetFileReqs::iterator it;
		for (it = FParent.outstanding.begin(); it != FParent.outstanding.end(); it++) 
			if (*it == &req) {
				FParent.outstanding.erase(it);
				break;
			}
		delete &req;
	}

	int remain = count;
	u8* dest = (u8*)buffer;
	while (remain) {
		int poss = 1024 - (Position&1023);
		int read = poss<remain?poss:remain;
//		printf("Rem %d Poss %d Rd %d Pos %d\n", remain, poss, read, Position);

		u8* chunk = FData[Position>>10];
		
		if (((FValidmask[Position>>13] >> ((Position>>10)&7)) & 1) == 0) printf("Fuck!");
		
		memcpy(dest, &chunk[Position & 1023], read);
		remain -= read;
		Position += read;
		dest += read;
	}


//	printf("Read %d/%d VM %d. ", Position, count, FValidmask[0]);
/*	int sample=0;
	for (i=0; i<4; i++) sample |= FData[Position+i] << ((3-i)*8);
	printf("Sample: %08X\n", sample);
*/	
//	Position += count;
	return count;
}

SylphNetFile::~SylphNetFile() {
	printf("(NetFile dtor)");
	int chunks = (FDatasize+1023)>>10; int i;
	for (i=0; i<chunks; ++i) delete[] FData[i];
	delete[] FData;
	delete[] FValidmask;

	SylphNetFiles::iterator it;
	for (it = FParent.children.begin(); it != FParent.children.end(); it++)
		if (*it == this) {
			FParent.children.erase(it);
			break;
		}
}

void SylphNetFile::IncomingData(int offset, int datasize, u8* data) {
//	printf("Rcv %d bytes at offset %d\n", datasize, offset);
	int voffset = offset >> 10;
	int vrange = ((datasize-1) >> 10) + 1;

	memcpy(FData[voffset], data, datasize); //TODO: Deal with non-KB chunks incoming!

	int i;
	for (i=voffset; i<(voffset+vrange); ++i)
		FValidmask[i>>3] |= (1 << (i&7));
	int sample=0;
	for (i=0; i<4; i++) sample |= data[i] << (i*8);
//	printf("Sample: %08X\n", sample);
}

SylphNetDataPlugin& NetData() {
	static SylphNetDataPlugin net;
	return net;
}

SylphNetFile::SylphNetFile(SylphNetDataPlugin& parent) : FData(0), FValidmask(0), FDatasize(0), FFileID(0), FParent(parent) {
}

