#ifndef __SYLPHDATANET_H
#define __SYLPHDATANET_H

#include <nds.h>
#include <string>
#include "SylphData.h"
#include "SylphActions.h"

class SylphNetFile;
class SylphNetDataPlugin;

class SylphNetFileMap {
	protected:
		volatile int FileID;
		int FileSize;
		std::string Filename;
		u16 Attributes;
	public:
		friend class SylphNetDataPlugin;
};

class SylphNetFileReq {
	protected:
		int Offset;
		int Range;
		volatile int Complete;
		SylphNetFile& FParent;
	protected:
		void Ping(bool resend);
	public:
		friend class SylphNetFile;
		friend class SylphNetDataPlugin;
		SylphNetFileReq(SylphNetFile& parent);
};

class SylphNetFileScan {
	protected:
		std::string FolderName;
		SylphFiles FFiles;
		bool FComplete;
	protected:
		void FIncoming(u16 PacketID, int datasize, u8* data);
	public:
		friend class SylphNetDataPlugin;
};

typedef std::vector<SylphNetFile*> SylphNetFiles;
typedef std::vector<SylphNetFileReq*> SylphNetFileReqs;
typedef std::vector<SylphNetFileMap*> SylphNetFileMaps;
typedef std::vector<SylphNetFileScan*> SylphNetFileScans;

class SylphNetDataPlugin: public SylphDataPlugin {
	protected:
		SylphNetFiles children;
		SylphNetFileReqs outstanding;
		SylphNetFileMaps maps;
		SylphNetFileScans scans;
		SylphFrame* FFrameSpin;
	protected:
		int GetData(std::string filename, SylphFile** file);
		SylphFiles* EnumFiles(std::string path);
	public:
		friend class SylphNetFile;
		void IncomingData(int FileID, int offset, int datasize, u8* data);
		void IncomingScan(std::string Filename, u16 PacketID, int datasize, u8* data);
		void FileStat(int FileID, int FileSize, u16 Attr, std::string Filename);
	
		SylphNetDataPlugin();
		~SylphNetDataPlugin();
		std::string PluginName();
		
		inline SylphFrame* getFrameSpin() { return FFrameSpin; }
		inline void setFrameSpin(SylphFrame* fs) { FFrameSpin = fs; }
};

class SylphNetFile: public SylphFile {
	protected:
		u8** FData;
		u8* FValidmask;
		int FDatasize;
		int FFileID;
		int FChunkCount;
		SylphNetDataPlugin& FParent;
	protected:
		void IncomingData(int offset, int datasize, u8* data);
	public:
		friend class SylphNetDataPlugin;
		friend class SylphNetFileReq;
		
		SylphNetFile(SylphNetDataPlugin& parent);
		~SylphNetFile();
		int Read(void* buffer, int count);
};

SylphNetDataPlugin& NetData();

#endif
