/*
     ``The contents of this file are subject to the Mozilla Public License
     Version 1.1 (the "License"); you may not use this file except in
     compliance with the License. You may obtain a copy of the License at
     http://www.mozilla.org/MPL/

     Software distributed under the License is distributed on an "AS IS"
     basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
     License for the specific language governing rights and limitations
     under the License.

     The Original Code is SylphDS (www.sylphds.net).

     The Initial Developer of the Original Code is Ficedula (web@sylphds.net).
*/

#include "SylphIRC.h"
#include "SylphUtil.h"

SylphIRC::SylphIRC() : FServer(""), FInfos(), FNames(), FNamesBuild() {
	FBufPos=0;
	FConnected=false;
	FChanData = NULL;
	FNameData = NULL;
}

void ClearSSL(StrStrList& list) {
	StrStrList::iterator p;
	for (p = list.begin(); p != list.end(); ++p)
		delete (*p).second;
}

SylphIRC::~SylphIRC() {
	ClearSSL(FNames);
	ClearSSL(FInfos);
	ClearSSL(FNamesBuild);
}

bool SylphIRC::Connect(std::string server, std::string nick, std::string username) {
	FSocket = socket(AF_INET,SOCK_STREAM,0);
	FServer = server;
	//printf("SylphIRC:Connect\n");
	struct hostent * the;
	the = gethostbyname(FServer.c_str());
	if (!the) {
		printf("SylphIRC: Couldn't lookup %s\n", FServer.c_str());
		return false;
	}
	FSockIn.sin_addr.s_addr=*((unsigned long *)(the->h_addr_list[0]));
    FSockIn.sin_port=htons(6667);
    connect(FSocket,(struct sockaddr *)&FSockIn,sizeof(FSockIn));
    int i=1;
    ioctl(FSocket,FIONBIO,&i); // set non-blocking
	FConnected = true;
	
	StringList user;
	user.push_back(username);
	user.push_back("NintendoDS");
	user.push_back("NintendoDS");
	user.push_back(username);
	FSend("USER", user);
	FSend("NICK", nick);
	FNick = nick;
	
	return true;
}

bool IsChan(std::string name) {
	return ((name.size() > 0) && ((name[0] == '#') || (name[0] == '&')));
}

StringList SplitParams(std::string input) {
	StringList result;
	while(input.size()) {
		if (input[0] == ':') {
			result.push_back(input.substr(1, input.size()-1));
			input = "";
		} else {
			int idx=0;
			while (input[idx] <= 32) idx++;
			int end=idx+1;
			while ((end < (int)input.size()) && (input[end] > 32)) end++;
			result.push_back(input.substr(idx, end-idx));
			while ((end < (int)input.size()) && (input[end] <= 32)) end++;
			input = input.substr(end, input.size()-end);
		}
	}
	return result;
}

void SylphIRC::FProcess(std::string input) {
	if (input == "") return;
//	printf(input.c_str());
//	printf("\n");
	
	int idx;
	std::string prefix(""), command, params;
	if (input[0] == ':') {
		for (idx=0; idx < (int)input.size(); ++idx)
			if (input[idx] <= 32) {
				prefix = input.substr(1, idx-1);
				input = input.substr(idx+1, input.size()-idx-1);
				break;
			} else if (idx == (int)(input.size()-1)) {
				prefix = input.substr(1, input.size()-1);
				input = "";
			}
	}
	int start=0;
	while ((start < (int)input.size()) && (input[start] <= 32)) start++;
	
	for (idx=start+1; idx < (int)input.size(); ++idx)
		if (input[idx] <= 32) {
			command = input.substr(start, idx-start);
			input = input.substr(idx+1, input.size()-idx-1);
			break;
		} else if (idx == (int)(input.size()-1)) {
			command = input.substr(start, input.size()-start);
			input = "";
		}
		
	params = input;
	
//	printf("%s %s %s", prefix.c_str(), command.c_str(), params.c_str());
	StringList pdata = SplitParams(params);
	if (command == "NOTICE")
		IRC_Notice(prefix, pdata);
	else if (command == "PING")
		IRC_Ping(prefix, pdata);
	else if (command == "QUIT")
		IRC_Quit(prefix, pdata);
	else if (command == "JOIN")
		IRC_Join(prefix, pdata);
	else if (command == "PART")
		IRC_Part(prefix, pdata);
	else if (command == "KICK")
		IRC_Kick(prefix, pdata);
	else if (command == "NICK")
		IRC_Nick(prefix, pdata);
	else if (command == "PRIVMSG")
		IRC_Privmsg(prefix, pdata);
	else if (command == "353")
		IRC_353(prefix, pdata);
	else if (command == "366")
		IRC_366(prefix, pdata);
}

void SylphIRC::Work() {
	int i;
	i=recv(FSocket, &FBuffer[FBufPos], 1024-FBufPos,0);
	if(i==0) FConnected = false;
	else if (i > 0) {
		int j, start;
		start = 0;
		for (j=FBufPos; j < (FBufPos+i-1); ++j) {
			if ((FBuffer[j] == 13) && (FBuffer[j+1] == 10)) {
				char* str = (char*)&FBuffer[start];
				FBuffer[j] = 0;
				FProcess(str);
				start = j+2;
			}
		}
		FBufPos += i;
		if (start) {
			memcpy(&FBuffer[0], &FBuffer[start], 1024-start);
			FBufPos -= start;
		}
	}
}

void SylphIRC::FSend(std::string command, StringList params) {
	std::string data = command;
	int i;
	for (i=0; i < (int)params.size()-1; ++i)
		data = data + ' ' + params[i];
	if (params.size())
		data = data + " :" + params[params.size()-1];
	data = data + "\x0D\x0A";
	//printf("Send: %s", data.c_str());
	send(FSocket, data.c_str(), data.size(), 0);
}

void SylphIRC::FSend(std::string command, std::string param) {
	StringList sl;
	sl.push_back(param);
	FSend(command, sl);
}

StringList* SylphIRC::getInfo(std::string name) {
	StrStrList::iterator p;
	p = FInfos.find(Lowercase(name));
	if (p != FInfos.end()) return p->second; else return NULL;
}

StringList* SylphIRC::getNames(std::string channel) {
	StrStrList::iterator p;
	p = FNames.find(Lowercase(channel));
	if (p != FNames.end()) return p->second; else return NULL;
}

StringList& DoEnsure(StrStrList& list, std::string key) {
	StrStrList::iterator p;
	key = Lowercase(key);
	p = list.find(key);
	if (p != list.end()) return *(p->second); else {
		StringList* sl = new StringList();
		list.insert(make_pair(key, sl));
		return *sl;
	}
}

StringList& SylphIRC::FEnsureNamesBuild(std::string channel) {
	return DoEnsure(FNamesBuild, channel);
}

StringList& SylphIRC::FEnsureNames(std::string channel) {
	return DoEnsure(FNames, channel);
}

StringList& SylphIRC::FEnsureInfo(std::string name) {
	return DoEnsure(FInfos, name);
}

std::string UserOf(std::string& prefix) {
	u32 idx = prefix.find('!');
	if (idx == std::string::npos) idx = prefix.size();
	return prefix.substr(0, idx);
}

std::string RawNick(std::string& nick) {
	if (nick.size() > 0)
		if (nick[0] == '@' || nick[0] == '%' || nick[0] == '&')
			return nick.substr(1, nick.size()-1);
	return nick;
}

void SylphIRC::IRC_Notice(std::string& prefix, StringList& params) {
	if (params.size() < 2) {
		printf("IRC_Notice: no param");
		return;
	}
	std::string source;
	std::string nick = UserOf(prefix);
	source = (prefix=="") ? "*SERVER" : IsChan(params[0]) ? params[0] : nick; //TODO: FIX!

	StringList& info = FEnsureInfo(source);
	info.push_back(nick + ": " + params[1]);
	while (info.size() > 64) info.erase(info.begin());
	if (FChanData) FChanData(source, info);
}

void SylphIRC::IRC_Quit(std::string& prefix, StringList& params) {
	if (params.size() < 1)
		printf("IRC_Quit: no param");
	else {
		StrStrList::iterator it;
		for (it=FNames.begin(); it!=FNames.end(); ++it) {
			StringList::iterator sli;
			for (sli = it->second->begin(); sli != it->second->end(); ++sli)
				if (RawNick(*sli) == UserOf(prefix)) {
					it->second->erase(sli);
					StringList& info = FEnsureInfo(it->first);
					info.push_back(UserOf(prefix) + " has quit: " + params[0]);
					while (info.size() > 64) info.erase(info.begin());
					if (FChanData) FChanData(it->first, info);
					if (FNameData) FNameData(it->first, *it->second);
					break; //iterator now invalidated
				}
		}
	}
}

void SylphIRC::IRC_Part(std::string& prefix, StringList& params) {
	if (params.size() < 1) 
		printf("IRC_Part: no param");
	else if (prefix == "")
		printf("IRC_Part: no prefix");
	else {
		std::string nick = UserOf(prefix);
		StringList& names = FEnsureNames(params[0]);
		StringList& info = FEnsureInfo(params[0]);
		StringList::iterator it;
		for (it = names.begin(); it != names.end(); ++it)
			if (RawNick(*it) == RawNick(nick)) {
				names.erase(it);
				info.push_back(RawNick(nick)+" left the channel");
				break;
			}
		while (info.size() > 64) info.erase(info.begin());
		if (FNameData) FNameData(params[0], names);
		if (FChanData) FChanData(params[0], info);
	}
}

void SylphIRC::IRC_Join(std::string& prefix, StringList& params) {
	if (params.size() < 1) 
		printf("IRC_Join: no param");
	else if (prefix == "")
		printf("IRC_Join: no prefix");
	else {
		std::string nick = UserOf(prefix);
		StringList& names = FEnsureNames(params[0]);
		StringList& info = FEnsureInfo(params[0]);
		names.push_back(nick);
		info.push_back(RawNick(nick)+" joined the channel");
		while (info.size() > 64) info.erase(info.begin());
		if (FNameData) FNameData(params[0], names);
		if (FChanData) FChanData(params[0], info);
	}
}

void SylphIRC::IRC_Kick(std::string& prefix, StringList& params) {
	if (params.size() < 2) 
		printf("IRC_Kick: no param");
	else if (prefix == "")
		printf("IRC_Kick: no prefix");
	else {
		StringList& names = FEnsureNames(params[0]);
		StringList& chan = FEnsureInfo(params[0]);
		std::string nick = params[1];
		StringList::iterator it;
		
		for (it = names.begin(); it != names.end(); ++it)
			if (RawNick(*it) == RawNick(nick)) {
				names.erase(it);
				break;
			}
		chan.push_back(UserOf(prefix) + " kicked " + nick);
		while (chan.size() > 64) chan.erase(chan.begin());
		if (FNameData) FNameData(params[0], names);
		if (FChanData) FChanData(params[0], chan);
	}
}

std::string MakePretty(std::string& user, std::string& message) {
	if (message.find(CTCP + "ACTION ") == 0)
		return ("~"+ user + " " + message.substr(8, message.size()-9));
	else if (message == CTCP + "VERSION"+CTCP)
		return (user + " requested your version");
	else if (message.find(CTCP + "PING ") == 0)
		return (user + " pinged you");
	else
		return (user + ": " + message);
}

void SylphIRC::IRC_Privmsg(std::string& prefix, StringList& params) {
	if (params.size() < 2) {
		printf("IRC_Privmsg: no param");
		return;
	}
	std::string source;
	std::string nick = UserOf(prefix);
	source = (prefix=="") ? "*SERVER" : IsChan(params[0]) ? params[0] : nick; //TODO: FIX!
	
	StringList& info = FEnsureInfo(source);
	if (params[1] == CTCP + "VERSION" + CTCP)
		Notice(nick, CTCP+"VERSION SylphIRCv"+SYLPH_IRC_VERSION+" NintendoDS www.sylphds.net"+CTCP);
	if (params[1].find(CTCP+"PING ")==0)
		Notice(nick, CTCP+"PING "+params[1].substr(6, params[1].size()-6)); //will include trailing CTCP already
	
	info.push_back(MakePretty(nick, params[1]));
	while (info.size() > 64) info.erase(info.begin());
	if (FChanData) FChanData(source, info);
}

void SylphIRC::IRC_353(std::string& prefix, StringList& params) {
	if (params.size() < 2)
		printf("IRC_353: no param");
	else {
		StringList& sl = FEnsureNamesBuild(params[params.size()-2]);
		StringList names = Tokenise(params[params.size()-1], " ");
		int i;
		for (i=0; i < (int)names.size(); ++i) {
			sl.push_back(names[i]);
		}
	}
}

void SylphIRC::IRC_366(std::string& prefix, StringList& params) {
	if (params.size() < 1)
		printf("IRC_366: no param");
	else {
		StringList& sl = FEnsureNamesBuild(params[params.size()-2]);
		StringList& names = FEnsureNames(params[params.size()-2]);
		names.clear();
		int i;
		for (i=0; i < (int)sl.size(); ++i) names.push_back(sl[i]); //assignment operator wasn't helping me here...
		sl.clear();
		if (FNameData) FNameData(params[params.size()-2], names);
	}
}

void SylphIRC::IRC_Nick(std::string& prefix, StringList& params) {
	if (params.size() < 1)
		printf("IRC_Nick: no param");
	else {
		StrStrList::iterator it;
		for (it=FNames.begin(); it!=FNames.end(); ++it) {
			int i;
			for (i=0; i < (int)it->second->size(); ++i)
				if (RawNick((*it->second)[i]) == UserOf(prefix)) {
					(*it->second)[i] = params[0];
					StringList& info = FEnsureInfo(it->first);
					info.push_back(UserOf(prefix) + " changed nick to " + params[0]);
					if (FChanData) FChanData(it->first, info);
					if (FNameData) FNameData(it->first, *it->second);
				}
		}
		if (FNick == prefix) FNick = params[0];
	}
}

void SylphIRC::IRC_Ping(std::string& prefix, StringList& params) {
	if (params.size() < 1)
		printf("IRC_Ping: no param");
	else 
		FSend("PONG", params[0]);
}

void SylphIRC::Notice(std::string target, std::string message) {
	StringList& sl = FEnsureInfo(target);
	sl.push_back(MakePretty(FNick, message));
	StringList params;
	params.push_back(target);
	params.push_back(message);
	FSend("NOTICE", params);
	if (FChanData) FChanData(target, sl);
}

void SylphIRC::PrivMsg(std::string target, std::string message) {
	StringList& sl = FEnsureInfo(target);
	sl.push_back(MakePretty(FNick, message));
	StringList params;
	params.push_back(target);
	params.push_back(message);
	FSend("PRIVMSG", params);
	if (FChanData) FChanData(target, sl);
}

void SylphIRC::Kick(std::string channel, std::string nick, std::string reason) {
	StringList params;
	params.push_back(channel);
	params.push_back(nick);
	params.push_back(reason);
	FSend("KICK", params);
}

void SylphIRC::Join(std::string channel) { 
	FSend("JOIN", channel); 
	/*StringList& sl = */ FEnsureInfo(channel);
}
