#ifndef __SYLPHIRC_H
#define __SYLPHIRC_H

#include <nds.h>
#include <string>
#include <map>

#include <dswifi9.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include "SylphUtil.h"
 
static const std::string CTCP("\x01");
static const std::string SYLPH_IRC_VERSION("0.05");

typedef std::map<std::string, StringList*> StrStrList;
typedef void(*TOnInfoChanged)(std::string, StringList&);

class SylphIRC {
	protected:
		std::string FServer;
		StrStrList FInfos;
		StrStrList FNames;
		StrStrList FNamesBuild;
		u8 FBuffer[1024];
		int FBufPos;
		int FSocket;
		struct sockaddr_in FSockIn;
		bool FConnected;
		TOnInfoChanged FChanData;
		TOnInfoChanged FNameData;
		std::string FNick;
	protected:
		void IRC_Notice(std::string& prefix, StringList& params);
		void IRC_Ping(std::string& prefix, StringList& params);
		void IRC_Quit(std::string& prefix, StringList& params);
		void IRC_Join(std::string& prefix, StringList& params);
		void IRC_Part(std::string& prefix, StringList& params);
		void IRC_Kick(std::string& prefix, StringList& params);
		void IRC_Privmsg(std::string& prefix, StringList& params);
		void IRC_353(std::string& prefix, StringList& params);
		void IRC_366(std::string& prefix, StringList& params);
		void IRC_Nick(std::string& prefix, StringList& params);
	protected:
		void FProcess(std::string input);
		void FSend(std::string command, StringList params);
		void FSend(std::string command, std::string param);
		StringList& FEnsureInfo(std::string name);
		StringList& FEnsureNames(std::string channel);
		StringList& FEnsureNamesBuild(std::string channel);
	public:
		SylphIRC();
		~SylphIRC();
		
		bool Connect(std::string server, std::string nick, std::string username);
		void Work();
		
		inline bool getConnected() { return FConnected; }
		StringList* getInfo(std::string name);
		StringList* getNames(std::string channel);
		inline StrStrList& getAllInfo() { return FInfos; }
		void Join(std::string channel);
		inline void CreateWindow(std::string channel) { FEnsureInfo(channel); }
		inline void Part(std::string channel) { FSend("PART", channel); }
		inline void Quit(std::string message) { FSend("QUIT", message); }
		inline void Nick(std::string nick) { FSend("NICK", nick); FNick = nick; }
		void Kick(std::string channel, std::string nick, std::string reason);
		void PrivMsg(std::string target, std::string message);
		void Notice(std::string target, std::string message);
		
		inline TOnInfoChanged getChanData() { return FChanData; }
		inline void setChanData(TOnInfoChanged cbk) { FChanData = cbk; }
		inline TOnInfoChanged getNameData() { return FNameData; }
		inline void setNameData(TOnInfoChanged cbk) { FNameData = cbk; }
		inline std::string getNick() { return FNick; }
};


#endif
