/*
     ``The contents of this file are subject to the Mozilla Public License
     Version 1.1 (the "License"); you may not use this file except in
     compliance with the License. You may obtain a copy of the License at
     http://www.mozilla.org/MPL/

     Software distributed under the License is distributed on an "AS IS"
     basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
     License for the specific language governing rights and limitations
     under the License.

     The Original Code is SylphDS (www.sylphds.net).

     The Initial Developer of the Original Code is Ficedula (web@sylphds.net).
*/

#include <nds.h>
#include "SylphNet.h"
#include "dswifi9.h"
#include "string.h"
#include "stdio.h"
#include "malloc.h"

static const char * assocstatus_strings[] = {
	"ASSOCSTATUS_DISCONNECTED", // not *trying* to connect
	"ASSOCSTATUS_AUTHENTICATING", // connecting...
	"ASSOCSTATUS_ASSOCIATING", // connecting...
	"ASSOCSTATUS_ASSOCIATED",	// Connected!
	"ASSOCSTATUS_CANNOTCONNECT", // error in connecting...
};

void Timer_50ms(void) {
   Wifi_Timer(50);
}

WifiSettings& GlobalWifi() {
	static WifiSettings settings;
	return settings;
}

const char* WifiAssocStatusStr(int status) {
	return assocstatus_strings[status];
}

void arm9_synctoarm7() { // send fifo message
   REG_IPC_FIFO_TX=0x87654321;
}
void arm9_fifo() { // check incoming fifo messages
   u32 value = REG_IPC_FIFO_RX;
   if(value == 0x87654321) Wifi_Sync();
}

void SylphNetInit() {
	int i;
	
    IPC->mailData=0;
    IPC->mailSize=0;
    REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR;
	u32 Wifi_pass= Wifi_Init(WIFIINIT_OPTION_USELED);
    REG_IPC_FIFO_TX=0x12345678;
    REG_IPC_FIFO_TX=Wifi_pass;

	*((volatile u16 *)0x0400010E) = 0;
	irqSet(IRQ_TIMER3, Timer_50ms);
	irqEnable(IRQ_TIMER3);
    irqSet(IRQ_FIFO_NOT_EMPTY, arm9_fifo);
    irqEnable(IRQ_FIFO_NOT_EMPTY);
    REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_RECV_IRQ;
    Wifi_SetSyncHandler(arm9_synctoarm7);

	*((volatile u16 *)0x0400010C) = -6553; // 6553.1 * 256 cycles = ~50ms;
	*((volatile u16 *)0x0400010E) = 0x00C2; // enable, irq, 1/256 clock

// set defaults
	GlobalWifi().connectAP.ssid_len=8;
	strcpy(GlobalWifi().connectAP.ssid,"ficedula");
	GlobalWifi().connectAP.rssi=0;
	GlobalWifi().connectAP.channel=6;
	GlobalWifi().connectAP.flags=0;
	GlobalWifi().WepKeyID=0;
	GlobalWifi().WepMode=0;
	GlobalWifi().dhcp=0;
	GlobalWifi().ipaddr=0xBD01A8C0;
	GlobalWifi().snmask=0x00FFFFFF;
	GlobalWifi().gateway=0x0101A8C0;
	GlobalWifi().dns1=0x0101A8C0;
	GlobalWifi().dns2=0;
	for(i=0;i<6;i++) GlobalWifi().connectAP.macaddr[i]=0; // clear mac if changing ssid.
	GlobalWifi().connectAP.base_rates[0]=0; // clear rates if changing ssid.

	iprintf("Wifilib Version 0.3\n");
	iprintf("Waiting for ARM7 to init..\n");
	while(Wifi_CheckInit()==0) swiWaitForVBlank();;
	iprintf("ARM7 Init Confirmed.\n");

	Wifi_EnableWifi();
	Wifi_ScanMode();
}

extern "C" {
void * sgIP_malloc(int size) { return malloc(size); }
void sgIP_free(void * ptr) { free(ptr); }
}

