/*
     ``The contents of this file are subject to the Mozilla Public License
     Version 1.1 (the "License"); you may not use this file except in
     compliance with the License. You may obtain a copy of the License at
     http://www.mozilla.org/MPL/

     Software distributed under the License is distributed on an "AS IS"
     basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
     License for the specific language governing rights and limitations
     under the License.

     The Original Code is SylphDS (www.sylphds.net).

     The Initial Developer of the Original Code is Ficedula (web@sylphds.net).
*/

#include <nds.h>
#include <string>

#include "SylphUDP.h"
#include "SylphDataNet.h"

void SylphUDP::Send(void* data, int datasize) {
	struct sockaddr_in sain;

	sain.sin_family = AF_INET;
	sain.sin_port = htons(FServerPort);
	sain.sin_addr.s_addr = FServerIP;
	sendto(FSocket, data, datasize, 0, (struct sockaddr *)&sain, sizeof(sain));
}

void SylphUDP::NOP() {
	//OK
}

void EncodeSS(std::string& s, void* output) {
	char* sdata = (char*)output;
	sdata+=2;
	s16 len = s.length();
	memcpy(output, &len, 2); //can't use pointer/direct write - alignment issues
	memcpy(sdata, s.c_str(), len);
}

u8* NewPacket(int size, SylphUDPOpcode opcode) {
	u8* data = new u8[size+3];
	data[0] = 0x8A;
	data[1] = 0x1F;
	data[2] = opcode;
	return data;
}

void SylphUDP::SendStringPacket(std::string& s, SylphUDPOpcode opcode) {
	int len = s.length()+2;
	u8* data = NewPacket(len, opcode);
	EncodeSS(s, data+3);
	Send(data, len+5);
	delete data;
}

void SylphUDP::Debug(std::string& text) {
	SendStringPacket(text, suDebug);
}

void SylphUDP::FileStat(std::string& filename) {
	SendStringPacket(filename, suFileStat);
//	printf("FileStat %s\n", filename.c_str());
}

void SylphUDP::FileScan(std::string& folder) {
	SendStringPacket(folder, suFileScan);
}

void SylphUDP::FileRead(int FileID, int requests, int* requestdata) {
	u8* data = NewPacket(6+8*requests, suFileRead);
	memcpy(&data[3], &FileID, 4);
	s16 numreq = requests/2;
	memcpy(&data[7], &numreq, 2);
	memcpy(&data[9], requestdata, 8*requests);
	Send(data, 9+8*requests+3);
	delete data;
}

void SylphUDP::Persist(int PID, int offset, int datasize, void* data) {
}

void SylphUDP::FileReset() {
	u8* packet = NewPacket(0, suFileReset);
	Send(packet, 3);
	delete packet;
}

void SylphUDP::RePing(int data) {
	u8* packet = NewPacket(4, suRePing);
	memcpy(packet+3, &data, 4);
	Send(packet, 7);
	delete packet;
}

void SylphUDP::PersistR(int PID, int offset, int datasize) {
}

void SylphUDP::Process() {
	struct sockaddr_in sain;

	int i=sizeof(sain);
	int sz;
	while ((sz=recvfrom(FSocket, &FRBuffer[0], 1700, 0, (struct sockaddr *)&sain, &i)) > 0) {
//		printf("UDP: Rcv %d bytes (opcode %d)\n", sz, FRBuffer[2]);
		if ((FRBuffer[0] == 0x9A) && (FRBuffer[1] == 0x1F)) { //is a sylph packet
			switch (FRBuffer[2]) { //opcode
				case 1:		{ //ping
								int data;
								memcpy(&data, &FRBuffer[3], 4);
								RePing(data);
								break;
							}
				case 2:		{ //filestat
								int FileID, FileSize;
								u16 Attrib, Len;
								memcpy(&FileID, &FRBuffer[3], 4);
								memcpy(&FileSize, &FRBuffer[7], 4);
								memcpy(&Attrib, &FRBuffer[11], 2);
								memcpy(&Len, &FRBuffer[13], 2);
								char* sdata = new char[Len+1];
								memcpy(sdata, &FRBuffer[15], Len);
								sdata[Len] = 0;
								printf("UDP:FS FN %s\n", sdata);
								NetData().FileStat(FileID, FileSize, Attrib, sdata);
								delete sdata;
								break;
							}
				case 3:		{ //filescan
								char tmpstr[1024];
								u16 wtmp, PacketID;
								memcpy(&wtmp, &FRBuffer[3], 2);
								memcpy(&tmpstr[0], &FRBuffer[5], wtmp);
								tmpstr[wtmp] = 0;
								std::string Filename(&tmpstr[0]);
								memcpy(&PacketID, &FRBuffer[5+wtmp], 2);
								NetData().IncomingScan(Filename, PacketID, sz-7-wtmp, &FRBuffer[7+wtmp]);
							}
				case 4:		{ //fileread
								int FileID, Offset;
								u16 DataSize;
								memcpy(&FileID, &FRBuffer[3], 4);
								memcpy(&Offset, &FRBuffer[7], 4);
								memcpy(&DataSize, &FRBuffer[11], 2);
								NetData().IncomingData(FileID, Offset, DataSize, &FRBuffer[13]);
								break;
							}
				default:	break;
			}
		}
	}
}

void SylphUDP::Connect() {
	struct sockaddr_in sain;

	FSocket = socket(AF_INET,SOCK_DGRAM,0);

	sain.sin_family=AF_INET;
	sain.sin_port=htons(9881);
	sain.sin_addr.s_addr=INADDR_ANY;
	bind(FSocket,(struct sockaddr *) &sain,sizeof(sain));
	int i=1;
	ioctl(FSocket,FIONBIO,&i);
}

SylphUDP& SUDP() {
	static SylphUDP udp;
	return udp;
}
