/*
     ``The contents of this file are subject to the Mozilla Public License
     Version 1.1 (the "License"); you may not use this file except in
     compliance with the License. You may obtain a copy of the License at
     http://www.mozilla.org/MPL/

     Software distributed under the License is distributed on an "AS IS"
     basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
     License for the specific language governing rights and limitations
     under the License.

     The Original Code is SylphDS (www.sylphds.net).

     The Initial Developer of the Original Code is Ficedula (web@sylphds.net).
*/

#include "SylphUtil.h"
#include <string>
#include <sstream>

std::string IntToStr(int val) {
	std::stringstream ss;
	ss << val;
	return ss.str();
}

std::string IPToStr(u32 IP) {
	std::stringstream ssIP;
	ssIP << (IP & 0xff) << '.' << ((IP>>8) & 0xff) << '.' << ((IP>>16) & 0xff) << '.' << ((IP>>24) & 0xff);
	return ssIP.str();
}

std::string Trim(const std::string& s) {

	int start, end;
	start = 0; end = s.size()-1;
	int i;
	for (i=0; i < (int)s.size(); ++i)
		if ((s[i] == 13) || (s[i] == 10) || (s[i] == 0) || (s[i] == 32) || (s[i] == 9))
			start++;
		else
			break;
	for (i = end; i > start; --i)
		if ((s[i] == 13) || (s[i] == 10) || (s[i] == 0) || (s[i] == 32) || (s[i] == 9))
			end--;
		else
			break;
//	printf("Trim %d/%d %s\n", start, end, s.c_str());
	if (start >= (int)s.size()) 
		return "";
	else
		return s.substr(start, end-start+1);
}

StringList LoadSL(SylphFile& file) {
	char* data = new char[file.FileSize+1];
	file.Seek(0, soStart);
	file.Read(data, file.FileSize);
	data[file.FileSize]=0;
	StringList result = LoadSL(data);
	delete[] data;
	return result;
}

StringList LoadSL(const std::string& data) {
	return Tokenise(data, "\n", true);
}

StringList Tokenise(const std::string& data, const std::string& breaker, bool DoTrim) {
	StringList* result = new StringList;
	u32 pos = 0;
	do {
		u32 newpos = data.find(breaker, pos);
		if (newpos == std::string::npos) {
			if (pos < data.size()) 
				if (DoTrim)
					result->push_back(Trim(data.substr(pos, data.size())));
				else
					result->push_back(data.substr(pos, data.size()));
			break;
		} else {
			if (DoTrim)
				result->push_back(Trim(data.substr(pos, newpos-pos)));
			else
				result->push_back(data.substr(pos, newpos-pos));
			pos = newpos+breaker.size();
		}
	} while (true);
	return *result;
}

std::string UnTokenise(const StringList& source, const std::string& breaker) { //TODO: Quoting
	int i;
	std::string result("");
	for (i=0; i < (int)source.size(); ++i) {
		result += source[i];
		if (i < (int)source.size()-1) result += breaker;
	}
	return result;
}

template<class T> int IndexOf(std::vector<T> src, T data) {
	int i;
	for (i=0; i < (int)src.size(); ++i)
		if (src[i] == data) return i;
	return -1;
}

StringListVals LoadSLV(StringList& src) {
	StringListVals slv;
	int i;
	for (i=0; i < (int)src.size(); ++i) {
		int pos = src[i].find("=");
		if (pos >= 0) slv[src[i].substr(0, pos)] = src[i].substr(pos+1, src[i].size()-pos-1);
	}
	return slv;
}

StringListVals LoadSLV(SylphFile& file) {
	StringList sl = LoadSL(file);
	return LoadSLV(sl);
}

std::string Lowercase(const std::string& s) {
	std::string lc;
	lc.resize( s.size() );
	u32 i;
	for(i=0; i < s.size(); ++i)
		lc[i] = tolower(s[i]);
	return lc;
}
