/*
     ``The contents of this file are subject to the Mozilla Public License
     Version 1.1 (the "License"); you may not use this file except in
     compliance with the License. You may obtain a copy of the License at
     http://www.mozilla.org/MPL/

     Software distributed under the License is distributed on an "AS IS"
     basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
     License for the specific language governing rights and limitations
     under the License.

     The Original Code is SylphDS (www.sylphds.net).

     The Initial Developer of the Original Code is Ficedula (web@sylphds.net).
*/

#include "SylphWidgets.h"

void SylphText::FClear() {
	int xL, yL, xH, yH;
	xL = FLastX >> 3;
	yL = FLastY >> 3;
	xH = (FLastX + FLastWidth) >> 3;
	yH = (FLastY + 12) >> 3; //TODO: TextHeight
	int y;
	for (y=yL; y<=yH; ++y)
		FBack.ClearTiles(xL, y, xH-xL+1);
}

/*
Valid values for FScrollState:
	0=Scrolling left
	1..60=Counting down to begin scrolling left
	100=Scrolling right
	101..160=Counting down to begin scrolling right
*/

void SylphText::FRender() {
	FFont.ClearHistory(this);
	FLastWidth = FFont.TextWidth(FText.c_str());
	if (FType == sttScrolling && FLastWidth > FWidth) {
		if (FScrollState > 0 && FScrollState < 100) FScrollState--;
		if (FScrollState > 100 && FScrollState < 200) FScrollState--;
		std::string clip = FText.substr(FScroll/16, FText.size()-FScroll/16);
		int length = FFont.DrawLimit(FWidth, clip, FBack, FX, FY, FColour, true, this);
		if (FScrollState == 0) {
			if (length == (int)clip.size()) FScrollState = 160; else FScroll++;
		} else if (FScrollState == 100) {
			if (--FScroll <= 0) FScrollState = 60;
		}
	} else
		FFont.DrawLimit(FWidth, FText, FBack, FX, FY, FColour, true, this);
	
	if (FLastWidth > FWidth) FLastWidth = FWidth;
	FLastX = FX;
	FLastY = FY;
}

void SylphText::FInit() {
	Reset();
	FClear();
	FRender();
}

void SylphText::FDraw() {
	if (FType == sttDynamic || FType == sttScrolling) {
		FClear();
		FRender();
	}
}

void SylphText::Reset() {
	FLastWidth = FFont.TextWidth(FText.c_str());
	FLastX = FX;
	FLastY = FY;
}

SylphText::SylphText(SylphBackground& FBack, std::string text, int x, int y, int width, int height, SylphFont& font, SylphTextType type, int colour)
	: SylphWidget(FBack), FText(text), FFont(font), FColour(colour) {
	FX = x;
	FY = y;
	FWidth = width;
	FHeight = height;
	FType = type;
	FScroll = 0;
	FScrollState = 0;
}

SylphText::~SylphText() {
	FFont.ClearHistory(this);
}


void SylphList::FInit() {
	Reset();
}

void SylphList::FDraw() {
	if (!FDirty) return;
	FFont.ClearHistory(this);
	int i;
	for (i = FLastY>>3; i <= (FLastY+FLastHeight)>>3; ++i)
		FBack.ClearTiles(FLastX>>3, i, ((FLastX+FLastWidth)>>3) - (FLastX>>3) + 1);
		
	int lines = (FHeight / 16);
	
	for (i = FTop; i < (FTop+lines) && i < (int)FItems.size(); ++i) {
		int colour=0;
		int x=FX+2;
		int y=FY+2+16*(i-FTop);
		int transparent=true;
		if (i == FPosition) {
			if (FHighlight & HIGHLIGHT_COLOUR) colour = FHighColour;
			if (FHighlight & HIGHLIGHT_INDENT) x += FHighIndent;
			if (FHighlight & HIGHLIGHT_SHADE) {
				FBack.SetTiles(x>>3, y>>3, (FWidth+FX-x)>>3, FHighShade);
				FBack.SetTiles(x>>3, 1+(y>>3), (FWidth+FX-x)>>3, FHighShade);
				transparent=false; //hope that text colour is same as shading colour...
			}
		}
		FFont.Draw(FItems[i], FBack, x, y, colour, transparent, this); 
	}
		
	FDirty=true;
	FLastX = FX;
	FLastY = FY;
	FLastWidth = FWidth;
	FLastHeight = FHeight;
}

void SylphList::setX(int x) {
	SylphWidget::setX(x);
	FDirty = true;
}

void SylphList::setY(int y) {
	SylphWidget::setY(y);
	FDirty = true;
}

void SylphList::setWidth(int w) {
	SylphWidget::setWidth(w);
	FDirty = true;
}

void SylphList::setHeight(int h) {
	SylphWidget::setHeight(h);
	FDirty = true;
}

SylphList::SylphList(SylphBackground& FBack, SylphFont& font, int x, int y, int width, int height) : SylphWidget(FBack), FFont(font) {
	FX = x;
	FY = y;
	FWidth = width;
	FHeight = height;
	Reset();
}

SylphList::~SylphList() {
	FFont.ClearHistory(this);
}

void SylphList::Reset() {
	FLastWidth = FWidth;
	FLastHeight = FHeight;
	FLastX = FX;
	FLastY = FY;
	FDirty = true;
}

void SylphList::setTop(int top) {
	if ((top < 0) || (FItems.size() == 0))
		FTop = 0;
	else if (FTop > ((int)FItems.size() - (FHeight/16)))
		FTop = FItems.size() - (FHeight/16);
	else
		FTop = top;
	FDirty=true; 
}

void SylphList::setPos(int pos) {
	if (FItems.size() == 0)
		FPosition = 0;
	else if (pos >= (int)FItems.size()) 
		FPosition = FItems.size()-1; 
	else if (pos < 0)
		FPosition = 0;
	else
		FPosition = pos;
	if (FTop > FPosition) setTop(FPosition);
	if (FTop < (FPosition - FHeight/16 + 1)) setTop(FPosition - FHeight/16 + 1);
	FDirty=true; 
}

SylphBorder::SylphBorder(SylphBackground &FBack, int x, int y, int w, int h, SylphFile& file) : SylphWidget(FBack) {
	FX = x;
	FY = y;
	FWidth = w;
	FHeight = h;
	SylphTiles tiles(file);
	int palr = FBack.ReservePal(tiles.numColours());
	FPalIndex = palr >> 16;
	FPalOffset = palr & 0xffff;
	tiles.LoadPalette(FBack, palr);
	u16* tiledata = FBack.TileAddress(0, 24); //just after default map
	tiles.LoadTiles(tiledata, FPalOffset);
}

SylphBorder::~SylphBorder() {
}

void SylphBorder::FInit() {
	int i;
	static const int Base=768;
	FBack.SetMap(FX, FY, Base, FPalIndex); //TL corner
	FBack.SetMap(FX+FWidth, FY, Base+1, FPalIndex); //TR corner
	FBack.SetMap(FX, FY+FHeight, Base+3, FPalIndex); //BL corner
	FBack.SetMap(FX+FWidth, FY+FHeight, Base+2, FPalIndex); //BR corner
	for (i=FX+1; i < (FX+FWidth); ++i) //top
		FBack.SetMap(i, FY, Base+5, FPalIndex);
	for (i=FX+1; i < (FX+FWidth); ++i) //bottom
		FBack.SetMap(i, FY+FHeight, Base+4, FPalIndex);
	for (i=FY+1; i < (FY+FHeight); ++i) //left
		FBack.SetMap(FX, i, Base+6, FPalIndex);
	for (i=FY+1; i < (FY+FHeight); ++i) //right
		FBack.SetMap(FX+FWidth, i, Base+7, FPalIndex);
	//printf("SylphBorder init\n");
}

void SylphBorder::FDraw() {
}

//SylphMemo

void SylphMemo::FClear() {
	//printf("Memo:Clear");
	int xL, yL, xH, yH;
	xL = FLastX >> 3;
	yL = FLastY >> 3;
	xH = (FLastX + FLastWidth) >> 3;
	yH = (FLastY + FLastHeight) >> 3;
	int y;
	for (y=yL; y<=yH; ++y)
		FBack.ClearTiles(xL, y, xH-xL+1); //copied from SylphText... 
}

void SylphMemo::FRender() {
	//printf("Memo:Render");
	FFont.ClearHistory(this);
	FLastX = FX;
	FLastY = FY;
	FLastWidth = FWidth;
	FLastHeight = FHeight;

	int i;
	for (i=0; (i < (FHeight/16)) && ((i+FTop) < (int)FItems.size()); ++i)
		FFont.DrawLimit(FWidth, FItems[i+FTop], FBack, FX, FY+16*i, FColour, true, this);
	//printf("OK");
}

void SylphMemo::FInit() {
	Reset();
//	FClear();
//	FRender();
}

void SylphMemo::FDraw() {
	if (FDirty) {
		FClear();
		FRender();
		FDirty = false;
	}
}

void SylphMemo::setTop(int top) {
	int maxtop = FItems.size() - (FHeight/16);
	FTop = (maxtop < 0 || top < 0) ? 0 : (maxtop>top ? top : maxtop);
	FDirty = true;
}

SylphMemo::SylphMemo(SylphBackground& back, SylphFont& font, int x, int y, int width, int height) : SylphWidget(back), FFont(font), FTop(0), FColour(0), FItems() {
	FDirty = true;
	FX = x;
	FY = y;
	FWidth = width;
	FHeight = height;
	FLastX = FX;
	FLastY = FY;
	FLastHeight = 0;
	FLastWidth = 0;
}

SylphMemo::~SylphMemo() {
	FFont.ClearHistory(this);
}

void SylphMemo::Reset() {
	FDirty = true;
	FItems.clear();
}

void SylphMemo::setX(int x) {
	SylphWidget::setX(x);
	FDirty = true;
}

void SylphMemo::setY(int y) {
	SylphWidget::setY(y);
	FDirty = true;
}

void SylphMemo::setWidth(int w) {
	SylphWidget::setWidth(w);
	FDirty = true;
}

void SylphMemo::setHeight(int h) {
	SylphWidget::setHeight(h);
	FDirty = true;
}

void SylphMemo::AddItems(StringList& sl) {
	int i;
	for (i=0; i < (int)sl.size(); ++i) Add(sl[i]);
}

void SylphMemo::Add(std::string s) {
	std::string part = s;
	do {
		int w = FFont.WrapTextAt(FWidth, part.c_str());
		if (w < (int)part.size()) { //search for a convenient word break
			int brk = w;
			while ((brk > 1) && (brk > (w-8))) {
				brk--;
				if (part[brk] == ' ') {
					w = brk;
					break;
				}
			}
		}
		FItems.push_back(part.substr(0, w));
		if (w < (int)part.size())
			part = part.substr(w, part.size()-1);
		else
			part = "";
	} while (part.size());
	FDirty = true;
}

void SylphButton::FInit() {
}

void SylphButton::FDraw() {
	if (FDirty) {
		//1: Clear
		if ((FX != FLastX) || (FY != FLastY) || (FWidth != FLastWidth) || (FHeight != FLastHeight)) {
			int xL, yL, xH, yH;
			xL = FLastX >> 3;
			yL = FLastY >> 3;
			xH = (FLastX + FLastWidth - 1) >> 3;
			yH = (FLastY + FLastHeight - 1) >> 3;
			int y;
			for (y=yL; y<=yH; ++y)
				FBack.ClearTiles(xL, y, xH-xL+1); //copied from SylphText... 
		}
		
		//2: Render
		int colour = FFocussed ? 1 : 0;
		
		FBack.DrawLine(FX, FY, FWidth, true, colour*4+3);
		FBack.DrawLine(FX, FY+FHeight-1, FWidth, true, colour*4+3);
		FBack.DrawLine(FX, FY, FHeight, false, colour*4+3);
		FBack.DrawLine(FX+FWidth-1, FY, FHeight, false, colour*4+3);
		FFont.Draw(FText, FBack, FX+2, FY+2, colour); //TODO: configurable colours
		
		FDirty = false;
		FLastX = FX;
		FLastY = FY;
		FLastWidth = FWidth;
		FLastHeight = FHeight;
	}
}

void SylphButton::doTouch(bool down) {
	FFocussed = down;
	FDirty = true;
}

bool SylphButton::canFocus() {
	return true;
}

void SylphButton::setFocussed(bool f) {
	SylphWidget::setFocussed(f);
	FDirty = true;
}

SylphButton::SylphButton(SylphBackground &back, SylphFont& font, int x, int y, int w, int h, std::string text) : SylphWidget(back), FText(text), FFont(font) {
	FX = x;
	FY = y;
	FWidth = w;
	FHeight = h;
	FLastX = x;
	FLastY = y;
	FLastWidth = w;
	FLastHeight = h;
	FDirty = true;
}

SylphButton::~SylphButton() {
}
