#ifndef __SYLPHWIDGETS_H
#define __SYLPHWIDGETS_H

#include "Sylph2D.h"
#include "SylphUtil.h"

static const int HIGHLIGHT_COLOUR = 0x01;
static const int HIGHLIGHT_INDENT = 0x02;
static const int HIGHLIGHT_SHADE = 0x04;

class SylphList: public SylphWidget {
	protected:
		SylphFont& FFont;
		StringList FItems;
		int FLastX, FLastY, FLastWidth, FLastHeight;
		int FPosition;
		int FTop;
		int FHighlight;
		bool FDirty;
		u8 FHighColour, FHighIndent, FHighShade;
	protected:
		void FInit();
		void FDraw();
	public:
		SylphList(SylphBackground& back, SylphFont& font, int x=0, int y=0, int width=0, int height=0);
		~SylphList();
		void Reset();
		
		inline StringList& Items() { return FItems; }
		inline void setItems(StringList& newitems) { FItems = newitems; }
		void setX(int x);
		void setY(int y);
		void setWidth(int w);
		void setHeight(int h);
		inline int getHighlight() { return FHighlight; }
		inline void setHighlight(int method) { FHighlight = method; }
		
		inline int getPos() { return FPosition; }
		void setPos(int pos);
		inline int getTop() { return FTop; }
		void setTop(int top);
		
		inline void setHighlights(int method, u8 colour, u8 indent, u8 shade) { FHighlight=method; FHighColour=colour; FHighIndent=indent; FHighShade=shade; FDirty=true; }
};

enum SylphTextType {sttNormal, sttDynamic, sttScrolling};

class SylphText: public SylphWidget {
	protected:
		std::string FText;
		SylphFont& FFont;
		int FColour;
		int FLastWidth; //to deal with changing text between frames
		int FLastX, FLastY;
		int FScroll;
		int FScrollState;
		SylphTextType FType;
	protected:
		void FClear();
		void FRender();
		void FInit();
		void FDraw();
	public:
		SylphText(SylphBackground& back, std::string text, int x, int y, int width, int height, SylphFont& font, SylphTextType type = sttNormal, int colour=0);
		~SylphText();
		
		void Reset();
		inline std::string getText() { return FText; }
		inline void setText(const std::string& s) { FText = s; }
		inline int getColour() { return FColour; }
		inline void setColour(int c) { FColour = c; }
		inline SylphTextType getType() { return FType; }
};

class SylphMemo: public SylphWidget {
	protected:
		SylphFont& FFont;
		int FLastHeight, FLastWidth; 
		int FLastX, FLastY;
		int FTop;
		int FColour;
		bool FDirty;
		StringList FItems;
	protected:
		void FClear();
		void FRender();
		void FInit();
		void FDraw();
	public:
		SylphMemo(SylphBackground& back, SylphFont& font, int x, int y, int width, int height);
		~SylphMemo();
	
		void Reset();
		inline StringList& Items() { return FItems; }
		void Add(std::string s);
		void AddItems(StringList& sl);
		inline int getColour() { return FColour; }
		inline void setColour(int c) { FColour = c; }
		void setX(int x);
		void setY(int y);
		void setWidth(int w);
		void setHeight(int h);
		inline int getTop() { return FTop; }
		void setTop(int top);
};

class SylphBorder: public SylphWidget {
	protected:
		int FPalIndex;
		int FPalOffset;
	protected:
		void FInit();
		void FDraw();
	public:
		SylphBorder(SylphBackground &back, int x, int y, int w, int h, SylphFile& file);
		~SylphBorder();
};

class SylphButton: public SylphWidget {
	protected:
		std::string FText;
		SylphFont& FFont;
		bool FDirty;
		int FLastWidth, FLastHeight, FLastX, FLastY;
	protected:
		void FInit();
		void FDraw();
	public:
		SylphButton(SylphBackground &back, SylphFont &font, int x, int y, int w, int h, std::string text);
		~SylphButton();
		
		inline std::string getText() { return FText; }
		inline void setText(std::string s) { FText = s; FDirty = true; }
		void doTouch(bool down);
		void setFocussed(bool f);
		bool canFocus();
};

#endif
