/*
     ``The contents of this file are subject to the Mozilla Public License
     Version 1.1 (the "License"); you may not use this file except in
     compliance with the License. You may obtain a copy of the License at
     http://www.mozilla.org/MPL/

     Software distributed under the License is distributed on an "AS IS"
     basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
     License for the specific language governing rights and limitations
     under the License.

     The Original Code is SylphDS (www.sylphds.net).

     The Initial Developer of the Original Code is Ficedula (web@sylphds.net).
*/

#include "ScrIRC.h"

#include "SylphData.h"
#include "SylphActions.h"
#include "SylphActionsDef.h"
#include "SylphNet.h"
#include "SylphDataCF.h"
#include "SylphUtil.h"
#include "SylphLayout.h"
#include "SylphWidgets.h"
#include "SylphIRC.h"

#include <sstream>

std::string build(""), serverurl("");
std::string source("");
bool shift=false, tempshift=false;

SylphMemo* memo;
SylphText* server;
SylphText* chans;
SylphText* nicks;
SylphText* status;
SylphText* text;
SylphIRC irc;
StringListVals config;

enum IState {isServer, isNick, isNull, isJoin, isPart, isChanMsg, isPrivMsg, isNickChg, isAction, isKick, isOpenPM};

IState state = isNull;

std::string StripSymbol(std::string src) {
	std::string rslt = src;
	while (rslt.size() && rslt[0] == '@') rslt = rslt.substr(1, rslt.size()-1);
	while (rslt.size() && rslt[0] == '%') rslt = rslt.substr(1, rslt.size()-1);
	while (rslt.size() && rslt[0] == '&') rslt = rslt.substr(1, rslt.size()-1);
	while (rslt.size() && rslt[0] == ' ') rslt = rslt.substr(1, rslt.size()-1);
	while (rslt.size() && rslt[0] == ',') rslt = rslt.substr(1, rslt.size()-1);
	return rslt;
}

void DoInfoChange(std::string which, StringList& info) {
	if (Lowercase(which) == Lowercase(source)) {
		memo->Items().clear();
		memo->AddItems(info);
		memo->setTop(999999);
		//source = which;
		chans->setText(source);
		if ((state == isNull) && (source != "")) state = isChanMsg;
		StringList* names = irc.getNames(source);
		if (names) nicks->setText(UnTokenise(*names, " "));
	}
}

void DoNameChange(std::string which, StringList& names) {
	if (Lowercase(which) == Lowercase(source)) {
		nicks->setText(UnTokenise(names, " "));
	}
}

void DoAction() {
	switch(state) {
		case isServer:	{
			serverurl = build;
			state = isNick;
			status->setText("Choose nick:");
			build = config["NICK"];
			break;
		}
		case isNick: {
			source = "*server";
			irc.Connect(serverurl, build, build);
			state = isNull;
			status->setText("Online");
			server->setText(serverurl);
			build="";
			break;
		}
		case isNull: {
			break;
		}
		case isJoin: {
			irc.Join(Trim(build));
			state = isNull;
			build="";
			break;
		}
		case isPart: {
			irc.Part(Trim(build));
			state = isNull;
			build="";
			break;
		}
		case isKick: {
			irc.Kick(source, Trim(build), "Kicked");
			state = isNull;
			build="";
			break;
		}
		case isNickChg: {
			irc.Nick(build);
			state = isNull;
			build="";
			break;
		}
		case isAction: {
			irc.PrivMsg(source, CTCP+"ACTION "+build+CTCP);
			state = isNull;
			build="";
			break;
		}
		case isChanMsg: 
		case isPrivMsg: {
			irc.PrivMsg(source, build);
			state = isNull;
			build="";
			break;
		}
		case isOpenPM: {
			irc.CreateWindow(build);
			build="";
			state = isNull;
			break;
		}
	}
	if ((state == isNull) && (source != "")) {
		state = isChanMsg;
		status->setText("");
	}
}

void FlipShift(SylphLayout& layout, SylphBackground& back) {
	shift = !shift;
	std::string fn = shift ? "sylphirc/keybh.sly" : "sylphirc/keybl.sly";
	std::string stat = status ? status->getText() : "";
	std::string focusn = layout.getName(layout.getFocus());
	layout.Clear();
	back.ClearWidgets();
	int i;
	for (i=0; i<24; ++i) back.ClearTiles(0, i, 32);
	
	SylphFile* file = Filer().GetData(fn);
	layout.Load(*file);
	delete file;

	status = dynamic_cast<SylphText*>(layout.Widget("status"));
	status->setText(stat);
	text = dynamic_cast<SylphText*>(layout.Widget("text"));
	text->setText(build);
	SylphWidget* focusw = layout.Widget(focusn);
	if (focusw) layout.setFocus(focusw);
}

void OnClick(SylphWidget* w, std::string name, SylphLayout& slayout, SylphBackground& sback) {
	if (name.find("keyb") == 0) {
		std::string key = name.substr(4, name.size()-4);
		if (key == "BK") {
			if (build.size() > 0) build = build.substr(0, build.size()-1);
		} else if (key == "RTN") {
			if (build != "") DoAction();
		} else if (key == "SHF") {
			FlipShift(slayout, sback);
			tempshift = true;
		} else if (key == "CAP") {
			FlipShift(slayout, sback);
		} else if (key == "SPC") {
			build += ' ';
		} else {
			build += key;
			if (tempshift) FlipShift(slayout, sback);
			tempshift = false;
		}
		text->setText(build);
	} else if ((name.find("cmd") == 0) && state != isServer && state != isNick) {
		std::string cmd = name.substr(3, name.size()-3);
		if (cmd == "Join") {
			build = "";
			state = isJoin;
			status->setText("Choose channel:");
		} else if (cmd == "Part") {
			build = "";
			state = isPart;
			status->setText("Choose channel:");
		} else if (cmd == "Nick") {
			build = "";
			state = isNickChg;
			status->setText("New nick:");
		} else if (cmd == "Action") {
			build = "";
			state = isAction;
			status->setText("Action:");
		} else if (cmd == "Kick") {
			build = "";
			state = isKick;
			status->setText("Enter nickname to kick:");
		} else if (cmd == "Quit") {
			irc.Quit("SylphIRC/DS");
			state = isNull;
			status->setText("OFFLINE");
		} else if (cmd == "OpenPM") {
			build = "";
			state = isOpenPM;
			status->setText("Enter nick to PM:");
		}
	}
}

void LaunchIRC() {
	SylphFrame framer;
	SylphInputAction input;
	framer.Register(input);
	
	irc.setChanData(DoInfoChange);
	irc.setNameData(DoNameChange);

	SylphGradientMap grad;

	lcdMainOnTop();

	VRAM_A_CR = VRAM_ENABLE | VRAM_A_MAIN_BG;
    videoSetMode(MODE_0_2D | DISPLAY_BG3_ACTIVE | DISPLAY_BG2_ACTIVE | DISPLAY_BG_EXT_PALETTE);
	VRAM_C_CR = VRAM_ENABLE | VRAM_C_SUB_BG;
	videoSetModeSub(MODE_0_2D | DISPLAY_BG3_ACTIVE | DISPLAY_BG2_ACTIVE | DISPLAY_BG_EXT_PALETTE);
	memset(BG_GFX, 0, 1024*128);
	memset(BG_GFX_SUB, 0, 1024*128);

	VRAM_F_CR = VRAM_ENABLE | VRAM_F_LCD; //enable VRAM bank F for direct writing
	grad.GenBankedPalettes(&VRAM_F[4096]);
	VRAM_F[0] = 0;
	VRAM_F[1] = 0;
	VRAM_F[2] = RGB15(4,4,4);
	VRAM_F[3] = RGB15(30,30,30);	//0-3 = Normal text colours
	VRAM_F[4] = 0;
	VRAM_F[5] = 0;
	VRAM_F[6] = RGB15(4,4,4);
	VRAM_F[7] = RGB15(30,0,0);		//4-7 = Red text
	VRAM_F[8] = RGB15(12, 12, 12);
	VRAM_F[9] = 0;
	VRAM_F[10] = RGB15(4,4,4);
	VRAM_F[11] = RGB15(30,30,30);	//8-11 = Shaded/highlight colours
	
	VRAM_H_CR = VRAM_ENABLE | VRAM_H_LCD;
	grad.GenBankedPalettes(&VRAM_H[4096*3]);
	VRAM_H[8192+0] = 0;
	VRAM_H[8192+1] = 0;
	VRAM_H[8192+2] = RGB15(4,4,4);
	VRAM_H[8192+3] = RGB15(30,30,30);	//0-3 = Normal text colours
	VRAM_H[8192+4] = 0;
	VRAM_H[8192+5] = 0;
	VRAM_H[8192+6] = RGB15(4,4,4);
	VRAM_H[8192+7] = RGB15(30,0,0);		//4-7 = Red text
	VRAM_H[8192+8] = RGB15(12, 12, 12);
	VRAM_H[8192+9] = 0;
	VRAM_H[8192+10] = RGB15(4,4,4);
	VRAM_H[8192+11] = RGB15(30,30,30);	//8-11 = Shaded/highlight colours
	
	VRAM_F_CR = VRAM_ENABLE | VRAM_F_BG_EXT_PALETTE | VRAM_OFFSET(1);
	VRAM_H_CR = VRAM_ENABLE | VRAM_H_SUB_BG_EXT_PALETTE;

	BG_CR[2] = BG_PRIORITY(2) | BG_256_COLOR | BG_TILE_BASE(1) | BG_MAP_BASE(0);
	BG_CR[3] = BG_PRIORITY(3) | BG_256_COLOR | BG_TILE_BASE(0) | BG_MAP_BASE(1);

	SUB_BG_CR[2] = BG_PRIORITY(2) | BG_256_COLOR | BG_TILE_BASE(1) | BG_MAP_BASE(0);
	SUB_BG_CR[3] = BG_PRIORITY(3) | BG_256_COLOR | BG_TILE_BASE(0) | BG_MAP_BASE(1);

	SylphBackground back((u16*)CHAR_BASE_BLOCK(1), (u16*)SCREEN_BASE_BLOCK(0));
	SylphBackground backp((u16*)CHAR_BASE_BLOCK(0), (u16*)SCREEN_BASE_BLOCK(1));
	back.InitPals(16, bgBankF);
	back.ReservePal(16); //for the data we already loaded above - this needs to be tidier :/

	SylphBackground sback((u16*)CHAR_BASE_BLOCK_SUB(1), (u16*)SCREEN_BASE_BLOCK_SUB(0));
	SylphBackground sbackp((u16*)CHAR_BASE_BLOCK_SUB(0), (u16*)SCREEN_BASE_BLOCK_SUB(1));
	sback.InitPals(16, bgExtendSub);
	sback.ReservePal(16); //for the data we already loaded above - this needs to be tidier :/

	framer.Register(back);
	framer.Register(backp);
	framer.Register(sback);
	framer.Register(sbackp);

	SylphFile* fontdata = Filer().GetData("sylphirc/font.bin");
	SylphFont font(*fontdata);
	fontdata->Seek(0, soStart);
	SylphFont fonttop(*fontdata);
	delete fontdata;

	back.InitUniqueMap(0);
	grad.GenBankedChars(backp, 208); //208 tiles = 13K ... we tuck the gradient characters
	grad.GenBankedMap(backp, 208); //into the top 3K exactly of the first 16KB block
	//The bottom part of the first 16KB is used for the background maps.
	sback.InitUniqueMap(0);
	grad.GenBankedChars(sbackp, 208); //208 tiles = 13K ... we tuck the gradient characters
	grad.GenBankedMap(sbackp, 208); //into the top 3K exactly of the first 16KB block
	//The bottom part of the first 16KB is used for the background maps.

	
	SylphLayout layout(back, fonttop);
	{
		SylphFile* file = Filer().GetData("sylphirc/memo.sly");
		layout.Load(*file);
		delete file;
	}

	SylphLayout slayout(sback, font);
	shift = true;
	FlipShift(slayout, sback);
	
	memo = dynamic_cast<SylphMemo*>(layout.Widget("chat"));
	chans = dynamic_cast<SylphText*>(layout.Widget("chans"));
	server = dynamic_cast<SylphText*>(layout.Widget("server"));
	nicks = dynamic_cast<SylphText*>(layout.Widget("nicks"));

	{
		SylphFile* file = Filer().GetData("sylphirc/sylphirc.cfg");
		config = LoadSLV(*file);
		delete file;
	}

	build = config["SERVER"];
	text->setText(build);
	state = isServer;
	status->setText("Enter server:");
	bool TouchKeyboard=true;
	
	//irc.Connect("aeris.mshome.net", "fice", "fice");
	while(true) {
/*	touchPosition txy;
	txy = touchReadXY();
	std::stringstream ss;
	ss << txy.x << " " << txy.y; //262 182		3790 3820
	status->setText(ss.str()); */

		if (state != isServer && state != isNick) {
			irc.Work();
			if (!irc.getConnected()) {
				printf("Disconnected!\n");
				break;
			}
		}

		framer.FrameStep(true);
		if (keysDown() & KEY_START) irc.Join(config["CHAN"]);
		if (keysDown() & KEY_B) memo->setTop(memo->getTop() + 1); 
		if (keysDown() & KEY_X) memo->setTop(memo->getTop() - 1);
		if (keysDown() & KEY_SELECT) {
			lcdSwap();
			TouchKeyboard = !TouchKeyboard;
		}
		if ((keysDown() & (KEY_L|KEY_R)) && (irc.getAllInfo().size() > 0))  {
			StrStrList::iterator it;
			it = irc.getAllInfo().find(source);
			if (it == irc.getAllInfo().end()) it = irc.getAllInfo().begin();
			else if ((keysDown() & KEY_L) && (it != irc.getAllInfo().begin())) --it;
			else if (keysDown() & KEY_R) {
				it++;
			}
			if (it == irc.getAllInfo().end()) it--;
			memo->Items().clear();
			memo->AddItems(*(it->second));
			memo->setTop(999999);
			source = it->first;
			chans->setText(source);
			if ((state == isNull) && (source != "")) state = isChanMsg;
			StringList* names = irc.getNames(source);
			if (names) {
				nicks->setText(UnTokenise(*names, " "));
			}
		}
		
		if (TouchKeyboard) {
			TouchResult r = slayout.ProcessTouch();
			if (r.type == ttDown && r.widget != NULL && r.widget->getFocussed()) {
				slayout.setFocus(r.widget);
			} else if (r.type == ttUp) {
				OnClick(r.widget, r.name, slayout, sback);
			}
		} else {
			if (keysDown() & KEY_TOUCH) {
				touchPosition txy;
				txy = touchReadXY();
				int tx = (txy.x-280) * 10 / 137;
				int ty = (txy.y-200) * 10 / 187;			
				std::string ctrl = fonttop.FindHistory(tx, ty);
				if (ctrl.size() > 0) build += StripSymbol(ctrl) + " ";
				text->setText(build);
//				status->setText(font.HistoryInfo());
			}
		}
		
		if ((keysDown() & KEY_A) && (slayout.getFocus())) {
			OnClick(slayout.getFocus(), slayout.getName(slayout.getFocus()), slayout, sback);
		}
		
		if (keysDown() & (KEY_UP | KEY_DOWN | KEY_LEFT | KEY_RIGHT)) {
			int vert = 0; int horiz = 0;
			if (slayout.getFocus()) {
				if (keysDown() & KEY_UP) vert=-1; else if (keysDown() & KEY_DOWN) vert=1;
				if (keysDown() & KEY_LEFT) horiz=-1; else if (keysDown() & KEY_RIGHT) horiz=1;
				SylphWidget* next = slayout.findNextWidget(*slayout.getFocus(), vert, horiz);
				if (next) {
					slayout.setFocus(next);
				}
			} else {
				slayout.setFirstFocus();
				if (slayout.getFocus()) slayout.getFocus()->setFocussed(true);
			}
		}
	}
}
