/*
     ``The contents of this file are subject to the Mozilla Public License
     Version 1.1 (the "License"); you may not use this file except in
     compliance with the License. You may obtain a copy of the License at
     http://www.mozilla.org/MPL/

     Software distributed under the License is distributed on an "AS IS"
     basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
     License for the specific language governing rights and limitations
     under the License.

     The Original Code is SylphDS (www.sylphds.net).

     The Initial Developer of the Original Code is Ficedula (web@sylphds.net).
*/

#include "SylphIPC.h"
#include <nds.h>

#ifdef ARM7
#include "SylphSound.h"
#include "SylphMikMod.h"
#endif

SylphIPC& SIPC() {
	static SylphIPC ipc;
	return ipc;
}

#ifdef ARM7

void SylphIPC::Process(SylphIPCCmd command, int param) {
	switch(command) {
		case ipHeartbeat:	Send(ipEcho, param); break;
		case ipEcho:		break; //TODO: Ack response, etc.
		case ipSoundSystem:	{
								Sound().CommandIPC(param);
								break;
							}
		case ipSoundBuffer:{
								Sound().SoundBuffer(param);
								break;
							}
		case ipDebug:		break; //TODO: Send to host
		case ipMikMod:		{
								MikMod().ProcessIPC(param);
								break;
							}
	}
}
#endif

#ifdef ARM9
#include "SylphSound9.h"

void SylphIPC::Process(SylphIPCCmd command, int param) {
	switch(command) {
		case ipHeartbeat:	Send(ipEcho, param); break;
		case ipEcho:		break; //TODO: Ack response, etc.
		case ipSoundBuffer:{	
								VoidFunctionPointer fp = Sound9().getFillBuffer();
								if (fp) fp();
								break; 
							}
		case ipMikMod:
		case ipSoundSystem:	break; //not implemented on ARM9
		case ipDebug:		break; //TODO: Send to host
	}
}
#endif

void SylphIPC::Send(SylphIPCCmd command, int param) {
	u32 data = (u8)command | (param << 8);
	while (REG_IPC_FIFO_CR & IPC_FIFO_SEND_FULL) { } //TODO: Timeout and report error if FIFO never available...?
	REG_IPC_FIFO_TX = data;
}

extern "C" void SylphIPCSend(u32 data) {
	SIPC().Send((SylphIPCCmd)(data & 0xff), data >> 8);
}

void FifoHandler(void) {
	while (!(REG_IPC_FIFO_CR & IPC_FIFO_RECV_EMPTY)) {
		u32 cmd = REG_IPC_FIFO_RX;
		SIPC().Process(SylphIPCCmd(cmd & 0xff), cmd>>8);
	}
}

void SylphIPCInit() {
	irqSet(IRQ_FIFO_NOT_EMPTY, FifoHandler);
	irqEnable(IRQ_FIFO_NOT_EMPTY);
	REG_IPC_FIFO_CR = REG_IPC_FIFO_CR | IPC_FIFO_RECV_IRQ | IPC_FIFO_ENABLE;
}
